//
//  ViewController.swift
//  macaw_test
//
//  Created by Chechkov Eugene on 5/8/17.
//  Copyright © 2017 xxx. All rights reserved.
//

import UIKit
import Macaw

class ViewController: UIViewController {
    
    @IBOutlet weak var mapView: UIView!
    
    private let mapName = "map_tr"
    private let mapHeight = 2587.0
    private let mapWidth = 2784.0
    private var svgView: SVGView?
    private var mapGroup: Macaw.Group?
    private var pathPolyline: Macaw.Shape?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view, typically from a nib.
        svgView = SVGView(frame: CGRect(x: 0, y: 0, width: mapHeight, height: mapWidth))
        svgView?.contentMode = .topLeft
        svgView?.fileName = mapName
        self.mapView.addSubview(svgView!)
        
        if let outerGroup = svgView?.node as? Group,
            let mainGroup = outerGroup.contents[0] as? Group,
            let symbolsGroup = mainGroup.contents[0] as? Group {
            
            mapGroup = symbolsGroup.contents[0] as? Group
            
            for node in mapGroup!.contents {
                addTapGestureToSVGElement(node: node, parentGroup: mapGroup!)
            }
    
        }
        
    }

    
    func addTapGestureToSVGElement(node: Macaw.Node, parentGroup: Group) {
        node.onTap({ (event) in
            if let node = event.node as? Shape {
                let coords = self.getCoordinatesOfNode(node: node)
                let newPathPolyline = Shape(
                    form: Macaw.Polyline(points: [0, 0, coords!.x, coords!.y]),
                    fill: nil,
                    stroke: Stroke(fill: LinearGradient(degree: 90, from: Color(val: 0x009eea), to: Color(val: 0xe30189)), width: 9))
                
                if self.pathPolyline != nil, let pathIndex = self.mapGroup!.contents.index(of: self.pathPolyline!) {
                        self.mapGroup?.contents[pathIndex] = newPathPolyline
                } else {
                    self.mapGroup?.contents.append(newPathPolyline)
                }
                self.pathPolyline = newPathPolyline
            }
        })
    }
    
    
    func getCoordinatesOfNode(node: Macaw.Node) -> (x: Double, y: Double)? {
        var x: Double? = nil
        var y: Double? = nil
        
        if node is Macaw.Shape {
            let shape = node as! Macaw.Shape
            if shape.form is Rect {
                let shapeRect = (shape.form as! Rect)
                x = shapeRect.x + shapeRect.w / 2
                y = shapeRect.y + shapeRect.h / 2
            } else if shape.form is RoundRect {
                let shapeRoundRect = (shape.form as! RoundRect)
                x = shapeRoundRect.rect.x + shapeRoundRect.rect.w / 2
                y = shapeRoundRect.rect.y + shapeRoundRect.rect.h / 2
            } else if shape.form is Circle {
                x = shape.form.bounds().x + shape.form.bounds().w / 2
                y = shape.form.bounds().y + shape.form.bounds().w / 2
            } else {
                return nil
            }
        } else if node is Macaw.Image {
            let image = node as! Macaw.Image
            x = image.place.dx
            y = image.place.dy
        }
        return (x != nil && y != nil) ? (x!, y!) : nil
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }


}

